use askama::Template;
use warp::{Filter, reply::Reply, reject::Rejection};
use warp::{reply::{with_status, with_header, html}, http::StatusCode};

use super::pages::ErrorPage;
use super::rejection::HttpReject;
use super::state::SharedState;

fn not_found_res(state: SharedState) -> Result<Box<dyn Reply>, Rejection> {
    let nf = ErrorPage::not_found(state.clone());
    Ok(
        Box::new(
            with_status(
                html(nf.render().map_err(|x| HttpReject::AskamaError(x))?),
                StatusCode::NOT_FOUND
            )
        )
    )
}

pub async fn get_resource((state, id): (SharedState, String)) -> Result<Box<dyn Reply>, Rejection> {
    if let Some(ref store) = state.config.resources {
        if let Some(ref res) = store.get(id) {
            Ok(
                Box::new(
                    with_header(
                        res.get().map_err(|x| HttpReject::StringError(x.to_string()))?,
                        "Content-Type",
                        res.mime.clone()
                    )
                )
            )
        } else {
            not_found_res(state)
        }
    } else {
        not_found_res(state)
    }
}

pub fn get_filter(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    warp::path!("resource" / "get" / String)
        .map(move |id| (state.clone(), id))
        .and_then(get_resource)
}

pub fn get_routes(state: SharedState) -> impl Filter<Extract = impl Reply, Error = Rejection> + Clone {
    get_filter(state)
}